==> rheolef_6.7.orig.tar.gz: rheolef-6.7/nfem/plib/characteristic.h <==
#ifndef _RHEO_CHARACTERISTIC_H
#define _RHEO_CHARACTERISTIC_H
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================
#include "rheolef/field.h"
#include "rheolef/quadrature.h"
#include "rheolef/basis_on_pointset.h"

namespace rheolef { 

// -------------------------------------------------------------------
// characteristic_on_quadrature: store the localization in mesh of the 
// quadrature point tacking: y = x + d(x) : that belongs in K as hat_y
// -------------------------------------------------------------------
template <class T, class M>
struct characteristic_on_quadrature_rep {
// allocators:
  characteristic_on_quadrature_rep (quadrature_option_type qopt1 = quadrature_option_type(quadrature_option_type::max_family,0))
  : _qopt         (qopt1),
    _quad         (),
    _piola_on_quad(),
    _ie2dis_ix    (),
    _hat_y        (),
    _yq           ()
  {}

  characteristic_on_quadrature_rep (const characteristic_on_quadrature_rep<T,M>& x)
  : _qopt         (x._qopt),
#ifdef TO_CLEAN
    _quad         (x._quad),
    _piola_on_quad(x._piola_on_quad),
    _ie2dis_ix    (x._ie2dis_ix),
    _hat_y        (x._hat_y),
    _yq           (x._yq)
#endif // TO_CLEAN
    _quad         (),
    _piola_on_quad(),
    _ie2dis_ix    (),
    _hat_y        (),
    _yq           ()
  {
    error_macro ("physical copy of characteristic_on_quadrature_rep may not happend"); 
  }

public:
// data:
  quadrature_option_type        _qopt;
  quadrature<T>                 _quad;
  basis_on_pointset<T>          _piola_on_quad;
  disarray<index_set,M>         _ie2dis_ix;
  disarray<point_basic<T>,M>    _hat_y;
  disarray<point_basic<T>,M>    _yq;
};
template <class T, class M>
class characteristic_on_quadrature : public smart_pointer<characteristic_on_quadrature_rep<T,M> > {
public:
// typedefs:
  typedef characteristic_on_quadrature_rep<T,M> rep;
  typedef smart_pointer<rep>                    base;
// allocator:
  characteristic_on_quadrature
    (quadrature_option_type qopt = quadrature_option_type(quadrature_option_type::max_family,0))
  : base (new_macro(rep(qopt)))
  {}
// data:
};
// -------------------------------------------------------------------
// characteristic_rep: store a list of characteristic_on_quadrature(s)
// together with the displacement field dh
// -------------------------------------------------------------------
template<class T, class M>
class characteristic_rep {
public:
  typedef std::map<std::string,characteristic_on_quadrature<T,M> > map_type;

// allocator:

  characteristic_rep(const field_basic<T,M>& dh)
  : _dh(dh),
    _coq_map()
  {}

// accesor:

  const field_basic<T,M>& get_displacement() const { return _dh; }

  const characteristic_on_quadrature<T,M>&
  get_pre_computed (
    const space_basic<T,M>&       Xh,
    const field_basic<T,M>&       dh,
    const quadrature_option_type& qopt) const;

// data:
protected:
  field_basic<T,M>  _dh;
  mutable map_type  _coq_map;
};

/*Class:characteristic
NAME: @code{characteristic} - the Lagrange-Galerkin method implemented
@cindex  Lagrange-Galerkin method
@cindex  method of characteristic
@cindex  quadrature formulae
@findex  riesz
@clindex space
@clindex geo

SYNOPSYS:
 The class characteristic implements the Lagrange-Galerkin method:
 It is the extension of the method of characteristic from the finite
 difference to the finite element context.
EXAMPLE:
@noindent
 Consider the bilinear form @code{lh} defined by
 @example
            /
            |
    lh(x) = | uh(x+dh(x)) v(x) dx
            |
            / Omega
 @end example
 where @code{dh} is a deformation vector field.
 The characteristic is defined by @code{X(x)=x+dh(x)}
 and the previous integral writes equivalently:
 @example
            /
            |
    lh(x) = | uh(X(x)) v(x) dx
            |
            / Omega
 @end example

 For instance, in Lagrange-Galerkin methods, 
 the deformation field @code{dh(x)=-dt*uh(x)} 
 where @code{uh} is the advection field and @code{dt} a time step.
 The following code implements the computation of lh:
 @example
  field dh = ...;
  field uh = ...;
  characteristic X (dh);
  test v (Xh);
  field lh = integrate (compose(uh, X)*v, qopt);
 @end example
 The Gauss-Lobatto quadrature formule is recommended for 
 Lagrange-Galerkin methods.
 The order equal to the polynomial order of Xh
 (order 1: trapeze, order 2: simpson, etc).
 Recall that this choice of quadrature formulae guaranties inconditional
 stability at any polynomial order.
 Alternative quadrature formulae or order can be used by using the 
 additional quadrature option argument to the @code{integrate} function @pxref{integrate algorithm}.
End: */

//<characteristic:
template<class T, class M = rheo_default_memory_model>
class characteristic_basic : public smart_pointer<characteristic_rep<T,M> > {
public:
  typedef characteristic_rep<T,M> rep;
  typedef smart_pointer<rep>      base;

// allocator:

  characteristic_basic(const field_basic<T,M>& dh);

// accesors:

  const field_basic<T,M>& get_displacement() const;

  const characteristic_on_quadrature<T,M>&
  get_pre_computed (
    const space_basic<T,M>&       Xh,
    const field_basic<T,M>&       dh,
    const quadrature_option_type& qopt) const;

};
typedef characteristic_basic<Float> characteristic;
//>characteristic:

template<class T, class M>
inline
characteristic_basic<T,M>::characteristic_basic (const field_basic<T,M>& dh)
 : base (new_macro(rep(dh)))
{
}
template<class T, class M>
inline
const field_basic<T,M>&
characteristic_basic<T,M>::get_displacement() const
{
  return base::data().get_displacement();
}
template<class T, class M>
inline
const characteristic_on_quadrature<T,M>&
characteristic_basic<T,M>::get_pre_computed (
    const space_basic<T,M>&       Xh,
    const field_basic<T,M>&       dh,
    const quadrature_option_type& qopt) const
{
  return base::data().get_pre_computed (Xh,dh,qopt);
}
// ===========================================================================
// the temporary class returned by compose (phi_h,X)
// ===========================================================================
template<class T, class M>
class field_o_characteristic {
public:

// allocator:

  field_o_characteristic(const field_basic<T,M>& uh, const characteristic_basic<T,M>& X)
    : _uh(uh), _X(X) {}

// accesor:

  const field_basic<T,M>&           get_field() const { return _uh; }
  const characteristic_basic<T,M>&  get_characteristic()  const { return _X; }

// data:
protected:
  field_basic<T,M>          _uh;
  characteristic_basic<T,M> _X;
};
#ifdef TO_CLEAN
// ===========================================================================
// compose (uh,X) returns a temporary:
// ===========================================================================
template<class T, class M>
inline
field_o_characteristic<T,M>
compose (const field_basic<T,M>& uh, characteristic_basic<T,M>& X)
{
  return field_o_characteristic<T,M>(uh, X);
}
// ===========================================================================
// riesz(Vh,compose (phi_h,X)) returns a field:
// => specialisation of the riesz function:
// ===========================================================================
template <class T, class M>
field_basic<T,M>
riesz (
    const space_basic<T,M>&             Xh,
    const field_o_characteristic<T,M>&  f,
    quadrature_option_type              qopt
       = quadrature_option_type(quadrature_option_type::max_family,0));
#endif // TO_CLEAN

}// namespace rheolef
#endif // _RHEO_CHARACTERISTIC_H

==> rheolef_7.0.orig.tar.gz: rheolef-7.0/nfem/plib/characteristic.h <==
#ifndef _RHEO_CHARACTERISTIC_H
#define _RHEO_CHARACTERISTIC_H
///
/// This file is part of Rheolef.
///
/// Copyright (C) 2000-2009 Pierre Saramito <Pierre.Saramito@imag.fr>
///
/// Rheolef is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.
///
/// Rheolef is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.
///
/// You should have received a copy of the GNU General Public License
/// along with Rheolef; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
/// 
/// =========================================================================
#include "rheolef/field.h"
#include "rheolef/quadrature.h"
#include "rheolef/basis_on_pointset.h"

namespace rheolef { 

// -------------------------------------------------------------------
// characteristic_on_quadrature: store the localization in mesh of the 
// quadrature point tacking: y = x + d(x) : that belongs in K as hat_y
// -------------------------------------------------------------------
template <class T, class M>
struct characteristic_on_quadrature_rep {
// allocators:
  characteristic_on_quadrature_rep (quadrature_option qopt1 = quadrature_option(quadrature_option::max_family,0))
  : _qopt         (qopt1),
    _quad         (),
    _piola_on_quad(),
    _ie2dis_ix    (),
    _hat_y        (),
    _yq           ()
  {}

  characteristic_on_quadrature_rep (const characteristic_on_quadrature_rep<T,M>& x)
  : _qopt         (x._qopt),
    _quad         (), // OK: DO NOT COPY !
    _piola_on_quad(),
    _ie2dis_ix    (),
    _hat_y        (),
    _yq           ()
  {
    error_macro ("physical copy of characteristic_on_quadrature_rep may not happened"); 
  }

public:
// data:
  quadrature_option        _qopt;
  quadrature<T>                 _quad;
  basis_on_pointset<T>          _piola_on_quad;
  disarray<index_set,M>         _ie2dis_ix;
  disarray<point_basic<T>,M>    _hat_y;
  disarray<point_basic<T>,M>    _yq;
};
template <class T, class M>
class characteristic_on_quadrature : public smart_pointer<characteristic_on_quadrature_rep<T,M> > {
public:
// typedefs:
  typedef characteristic_on_quadrature_rep<T,M> rep;
  typedef smart_pointer<rep>                    base;
// allocator:
  characteristic_on_quadrature
    (quadrature_option qopt = quadrature_option(quadrature_option::max_family,0))
  : base (new_macro(rep(qopt)))
  {}
// data:
};
// -------------------------------------------------------------------
// characteristic_rep: store a list of characteristic_on_quadrature(s)
// together with the displacement field dh
// -------------------------------------------------------------------
template<class T, class M>
class characteristic_rep {
public:
  typedef std::map<std::string,characteristic_on_quadrature<T,M> > map_type;

// allocator:

  characteristic_rep(const field_basic<T,M>& dh);

// accesor:

  const field_basic<T,M>& get_displacement() const { return _dh; }

  const characteristic_on_quadrature<T,M>&
  get_pre_computed (
    const space_basic<T,M>&       Xh,
    const field_basic<T,M>&       dh,
    const quadrature_option& qopt) const;

// data:
protected:
  field_basic<T,M>  _dh;
  mutable map_type  _coq_map;
};

/*Class:characteristic
NAME: @code{characteristic} - the Lagrange-Galerkin method implemented
@cindex  Lagrange-Galerkin method
@cindex  method of characteristic
@cindex  quadrature formulae
@findex  riesz
@clindex space
@clindex geo

SYNOPSIS:
 The class characteristic implements the Lagrange-Galerkin method:
 It is the extension of the method of characteristic from the finite
 difference to the finite element context.
EXAMPLE:
@noindent
 Consider the bilinear form @code{lh} defined by
 @example
            /
            |
    lh(x) = | uh(x+dh(x)) v(x) dx
            |
            / Omega
 @end example
 where @code{dh} is a deformation vector field.
 The characteristic is defined by @code{X(x)=x+dh(x)}
 and the previous integral writes equivalently:
 @example
            /
            |
    lh(x) = | uh(X(x)) v(x) dx
            |
            / Omega
 @end example

 For instance, in Lagrange-Galerkin methods, 
 the deformation field @code{dh(x)=-dt*uh(x)} 
 where @code{uh} is the advection field and @code{dt} a time step.
 The following code implements the computation of lh:
 @example
  field dh = ...;
  field uh = ...;
  characteristic X (dh);
  test v (Xh);
  field lh = integrate (compose(uh, X)*v, qopt);
 @end example
 The Gauss-Lobatto quadrature formule is recommended for 
 Lagrange-Galerkin methods.
 The order equal to the polynomial order of Xh
 (order 1: trapeze, order 2: simpson, etc).
 Recall that this choice of quadrature formulae guaranties inconditional
 stability at any polynomial order.
 Alternative quadrature formulae or order can be used by using the 
 additional quadrature option argument to the @code{integrate} function @pxref{integrate algorithm}.
End: */

//<characteristic:
template<class T, class M = rheo_default_memory_model>
class characteristic_basic : public smart_pointer<characteristic_rep<T,M> > {
public:
  typedef characteristic_rep<T,M> rep;
  typedef smart_pointer<rep>      base;

// allocator:

  characteristic_basic(const field_basic<T,M>& dh);

// accesors:

  const field_basic<T,M>& get_displacement() const;

  const characteristic_on_quadrature<T,M>&
  get_pre_computed (
    const space_basic<T,M>&       Xh,
    const field_basic<T,M>&       dh,
    const quadrature_option& qopt) const;

};
typedef characteristic_basic<Float> characteristic;
//>characteristic:

template<class T, class M>
inline
characteristic_basic<T,M>::characteristic_basic (const field_basic<T,M>& dh)
 : base (new_macro(rep(dh)))
{
}
template<class T, class M>
inline
const field_basic<T,M>&
characteristic_basic<T,M>::get_displacement() const
{
  return base::data().get_displacement();
}
template<class T, class M>
inline
const characteristic_on_quadrature<T,M>&
characteristic_basic<T,M>::get_pre_computed (
    const space_basic<T,M>&       Xh,
    const field_basic<T,M>&       dh,
    const quadrature_option& qopt) const
{
  return base::data().get_pre_computed (Xh,dh,qopt);
}
// ===========================================================================
// the temporary class returned by compose (phi_h,X)
// ===========================================================================
template<class T, class M>
class field_o_characteristic {
public:

// allocator:

  field_o_characteristic(const field_basic<T,M>& uh, const characteristic_basic<T,M>& X)
    : _uh(uh), _X(X) {}

// accesor:

  const field_basic<T,M>&           get_field() const { return _uh; }
  const characteristic_basic<T,M>&  get_characteristic()  const { return _X; }

// data:
protected:
  field_basic<T,M>          _uh;
  characteristic_basic<T,M> _X;
};

}// namespace rheolef
#endif // _RHEO_CHARACTERISTIC_H
